/**
 * @file
 * Attaches behaviors for Menu Migration help page.
 */

(function (Drupal) {
  /**
   * Provides javascript for the help page vertical tabs.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.menuMigrationHelpPage = {
    attach(context) {
      const tabNavItems = context.querySelectorAll('.mm-tabs-nav li');
      const tabPanes = context.querySelectorAll('.mm-tab-pane');

      tabNavItems.forEach((item) => {
        item.addEventListener('click', function () {
          const tabIndex = this.dataset.tab;

          // Remove 'active' class from all nav items and 'hidden' from all
          // panes.
          tabNavItems.forEach((nav) => nav.classList.remove('active'));
          tabPanes.forEach((pane) => pane.classList.add('hidden'));

          // Add 'active' class to the clicked nav item.
          this.classList.add('active');

          // Show the corresponding tab pane.
          const targetPane = context.querySelector(
            `.mm-tab-pane[data-tab-content="${tabIndex}"]`,
          );
          if (targetPane) {
            targetPane.classList.remove('hidden');
          }
        });
      });

      // Initialize: ensure the first tab is active and visible on load.
      if (tabNavItems.length > 0) {
        tabNavItems[0].classList.add('active');
      }
      if (tabPanes.length > 0) {
        tabPanes[0].classList.remove('hidden');
      }
    },
  };
})(Drupal);
