<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\Kernel;

/**
 * Tests the menu_item_extras contrib module integration.
 *
 * @group menu_migration
 */
class MenuItemExtrasModuleTest extends MenuMigrationKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'link',
    'field',
    'menu_link_content',
    'menu_item_extras',
    'menu_migration',
  ];

  /**
   * The field name that we are going to work with.
   *
   * @var string
   */
  protected string $fieldName = 'field_test_extras';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $entityType = 'menu_link_content';
    $this->entityTypeManager()->getStorage('field_storage_config')->create([
      'field_name' => $this->fieldName,
      'entity_type' => $entityType,
      'type' => 'string',
      'cardinality' => 1,
    ])->save();
    $this->entityTypeManager()->getStorage('field_config')->create([
      'field_name' => $this->fieldName,
      'entity_type' => $entityType,
      'bundle' => $entityType,
      'label' => 'Test Extra Field',
    ])->save();
  }

  /**
   * Tests the export and import of fields created with menu_item_extras module.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testMenuItemExtras(): void {
    $link = $this->createMenuLink([
      'title' => 'Menu Item Extras Title',
      'description' => 'Menu Item Extras Description',
      'link' => ['uri' => 'internal:/test-path'],
      'menu_name' => $this->menu->id(),
      $this->fieldName => 'Some test value',
      'view_mode' => 'default',
    ]);
    $link->save();
    $this->exportMenus();
    $this->importMenus();

    $menuTree = $this->loadMenuTree();
    $menuTreeElement = reset($menuTree);
    $this->assertMenuLinkElementEquals($link, $menuTreeElement, ['view_mode', $this->fieldName]);
  }

}
