<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\Kernel;

/**
 * Tests the menu_per_role contrib module integration.
 *
 * @group menu_migration
 */
class MenuPerRoleModuleTest extends MenuMigrationKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'link',
    'menu_link_content',
    'menu_per_role',
    'menu_migration',
  ];

  /**
   * Tests the export and import of menu items using the menu_per_role module.
   */
  public function testMenuPerRole(): void {
    $link = $this->createMenuLink([
      'title' => 'Menu Item Title',
      'description' => 'Menu Item Description',
      'link' => ['uri' => 'internal:/some-path'],
      'menu_name' => $this->menu->id(),
      'menu_per_role__show_role' => [
        ['target_id' => 'anonymous'],
      ],
      'menu_per_role__hide_role' => [
        ['target_id' => 'authenticated'],
      ],
    ]);
    $link->save();

    $this->assertTrue($link->hasField('menu_per_role__show_role'), 'The menu link has the "menu_per_role__show_role" field');
    $this->assertTrue($link->hasField('menu_per_role__hide_role'), 'The menu link has the "menu_per_role__hide_role" field');

    $this->exportMenus();
    $this->importMenus();

    $menuTree = $this->loadMenuTree();
    $importedLinkElement = reset($menuTree);
    $this->assertMenuLinkElementEquals($link, $importedLinkElement, [
      'menu_per_role__show_role',
      'menu_per_role__hide_role',
    ]);
  }

}
