<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\Kernel;

use Drupal\language\Entity\ConfigurableLanguage;

/**
 * Tests the translatable_menu_link_uri contrib module integration.
 *
 * @group menu_migration
 */
class TranslatableMenuLinkUriModuleTest extends MenuMigrationKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'link',
    'language',
    'menu_link_content',
    'menu_migration',
    'translatable_menu_link_uri',
  ];

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('menu_link_content');
    ConfigurableLanguage::createFromLangcode('es')->save();
  }

  /**
   * Tests the export/import menu items that have translatable URIs.
   */
  public function testTranslatableMenuLinkUri(): void {
    $link = $this->createMenuLink([
      'title' => 'Original Title',
      'description' => 'Original Description',
      'link' => ['uri' => 'internal:/original-link'],
      'menu_name' => $this->menu->id(),
    ]);
    $link->save();

    $linkTranslation = $link->addTranslation('es', [
      'title' => 'Spanish Title',
      'description' => 'Spanish Description',
      'link_override' => ['uri' => 'internal:/spanish-link'],
    ]);
    $linkTranslation->save();

    $this->assertTrue($link->hasField('link_override'), 'The menu link has the link_override field.');
    $this->assertTrue($linkTranslation->hasField('link_override'), 'The mnu link translation has the link_override field.');

    $this->exportMenus();
    $this->importMenus();

    /** @var \Drupal\menu_link_content\MenuLinkContentInterface[] $importedLinks */
    $importedLinks = $this->menuLinkContentStorage()->loadByProperties(['menu_name' => $this->menu->id()]);
    $this->assertCount(1, $importedLinks, 'The menu has the expected number of menu items.');
    $importedLink = reset($importedLinks);
    $this->assertTrue($importedLink->hasTranslation('es'), 'The imported menu link has the ES translation.');
    $importedLinkTranslation = $importedLink->getTranslation('es');
    $this->assertBasicMenuLinkContentEquals($link, $importedLink);
    $this->assertBasicMenuLinkContentEquals($linkTranslation, $importedLinkTranslation);
    $this->assertEmpty($importedLink->get('link_override')->getValue()[0]['uri']);
    $this->assertEquals($linkTranslation->get('link_override')->getValue()[0]['uri'], $importedLinkTranslation->get('link_override')->getValue()[0]['uri']);
  }

}
