<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\Kernel;

use Drupal\language\Entity\ConfigurableLanguage;

/**
 * Tests the import and export of translatable menu items.
 *
 * @group menu_migration
 */
class TranslatableMenuLinksTest extends MenuMigrationKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'link',
    'language',
    'menu_link_content',
    'menu_migration',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('menu_link_content');
    ConfigurableLanguage::createFromLangcode('es')->save();
    ConfigurableLanguage::createFromLangcode('fr')->save();
  }

  /**
   * Tests the export and import of translatable menu items.
   */
  public function testMenuItemsTranslation(): void {
    $hierarchy = $this->createSimpleMenuHierarchy();
    foreach ($hierarchy as $item) {
      $item->addTranslation('es', [
        'title' => $item->getTitle() . ' (ES)',
        'description' => $item->getDescription() . ' (ES)',
      ])->save();
      $item->addTranslation('fr', [
        'title' => $item->getTitle() . ' (FR)',
        'description' => $item->getDescription() . ' (FR)',
      ])->save();
    }

    $this->exportMenus();
    $this->importMenus();

    $menuTree = $this->loadMenuTree();
    $importedHierarchy['parent'] = reset($menuTree);
    $importedHierarchy['child'] = reset($importedHierarchy['parent']->subtree);
    $importedHierarchy['sub_child'] = reset($importedHierarchy['child']->subtree);
    foreach ($hierarchy as $key => $expectedLink) {
      $importedLink = $this->loadMenuLinkFromTreeElement($importedHierarchy[$key]);
      $this->assertTrue($importedLink->hasTranslation('es'), 'The imported menu link has the ES translation.');
      $this->assertTrue($importedLink->hasTranslation('fr'), 'The imported menu link has the FR translation.');
      $this->assertBasicMenuLinkContentEquals($expectedLink->getTranslation('es'), $importedLink->getTranslation('es'));
      $this->assertBasicMenuLinkContentEquals($expectedLink->getTranslation('fr'), $importedLink->getTranslation('fr'));
    }

    // Test removal of one translation.
    $parentLink = $this->loadMenuLinkFromTreeElement($importedHierarchy['parent']);
    $parentLink->removeTranslation('es');
    $parentLink->save();

    $this->exportMenus();
    $this->importMenus();

    $menuTree = $this->loadMenuTree();
    $importedParent = reset($menuTree);
    $importedParentLink = $this->loadMenuLinkFromTreeElement($importedParent);
    $this->assertFalse($importedParentLink->hasTranslation('es'), 'The imported menu link does not have the ES translation.');
    $this->assertTrue($importedParentLink->hasTranslation('fr'), 'The imported menu link has the FR translation.');
  }

}
